#!/usr/bin/python

import os, re
import pickle
from alin import *

#DEFAULT_PATH = "/home/projects/alba-em/deviceslib/"

class Generator():
	def __init__(self):
		self.fread = []
		self.vendorID = None
		self.product = None
		self.wboutput = {}
		
		self.default_path = os.path.dirname(alin.__file__)+"/deviceslib/"
		if not os.path.exists(self.default_path):
			os.makedirs(self.default_path)

	def readWBFile(self, filename):
		try:
			with open(filename) as f:
				self.fread = [" ".join(l.split()) for l in f]
		except Exception, e:
			print "Error while opening in wb file: %s"%str(e)
			return
		
	def getInputVendorID(self):
		self._vendorID = None
		while not self._vendorID:
			self.vendorID = raw_input ("\nEnter VENDOR ID code (hex format, max. 8 bytes)" \
				"(i.e: a1ba for Alba or ce42 for CERN) or enter 'Q' to quit: ")
			if self.vendorID.lower() == "q":
				print "\nProgram aborted!!!\n"
				return None
			try:
				self.vendorID = int(self.vendorID.lower(),16)
				if self._vendorID > 0xffffffffffffffff:
					print "Number exceeds maximum value. Please enter hexadecimal " \
						"(max. 8 bytes) vendor ID value!!\n "
					self.vendorID = None
				else:
					return self.vendorID
			except:
				print "Incorrect number format!!! Please enter hexadecimal vendor ID value!!\n"
	
	def getInputProduct(self):
		self.product = None
		while not self.product:
			self.product = raw_input ("\nEnter PRODUCT (hex format, max 4 bytes)" \
				"or enter 'Q' to quit: ")
			if self.product.lower() == "q":
				print "\nProgram aborted!!!\n"
				return None
			try:
				self.product = int(self.product.lower(),16)
				if self.product > 0xffffffff:
					print "Number exceeds maximum value. Please enter hexadecimal "\
						"(max. 4 bytes) product value!!\n"
					self.product = None
				else:
					return self.product
			except:
				print "Incorrect number format!!! Please enter hexadecimal product value!!\n"
	
	def processWBData(self):
		self.wboutput = {}
		offset = 0
		l = 0
		while l<len(self.fread):
			# Read device description
			if "name = " in self.fread[l] or "description =" in self.fread[l]:
				if self.fread[l].split()[0] not in self.wboutput.keys():
					self.wboutput[self.fread[l].split()[0]] = re.search('"(.*)"',self.fread[l].split("=")[1]).group(1)
				else:
					self.wboutput[self.fread[l].split()[0]] += re.search('"(.*)"',self.fread[l].split("=")[1]).group(1)
					
			# Read registers				
			elif "reg {" in self.fread[l]:
				reg_name = ""
				bit_position = 0
				l += 1
				while "};" not in self.fread[l]:

					if "align" in self.fread[l]:
						offset += int(self.fread[l].split("=")[1].replace(";",""))
					elif "prefix" in self.fread[l]:
						reg_name += re.search('"(.*)"',self.fread[l].split("=",1)[1]).group(1)
					elif "field {" in self.fread[l]:
						l += 1

						size = 0
						type = ""
						description = ""
						field_name = ""
						store_reg = True
						while "};" not in self.fread[l]:
							if "prefix =" in self.fread[l]:
								if "reserved" in self.fread[l].lower():
									store_reg = False							
								else:
									field_name = re.search('"(.*)"',self.fread[l].split("=",1)[1]).group(1)
									store_reg = True
							elif "access_bus =" in self.fread[l]:
								access = self.fread[l].split("=")[1].replace(";","")
							elif "type = " in self.fread[l] and "BIT" in self.fread[l]:
								size = 1
							elif "size = " in self.fread[l]:
								size = int(self.fread[l].split("=")[1].replace(";",""))
							elif "description = " in self.fread[l]:
								description = re.search('"(.*)"',self.fread[l].split("=",1)[1]).group(1)
							elif "align" in self.fread[l]:
								bit_offset = int(self.fread[l].split("=")[1].replace(";",""))
								if bit_position % bit_offset: 
									ad = bit_offset - (bit_position % bit_offset)
									bit_position += ad
							l +=1

						if store_reg:
							data = {}
							if field_name != "":
								reg = reg_name+"_"+field_name
							else:
								reg = reg_name
							data[reg]={}
							data[reg]["bit_position"] = bit_position
							data[reg]["size"] = size
							data[reg]["access"] = access
							data[reg]["description"] = description
							bit_position += size
							data[reg]["address"] = offset

							if "regs" not in self.wboutput.keys():
								self.wboutput["regs"]=[data]
							else:
								self.wboutput["regs"].append(data)
							
					l+=1
				
				offset +=1
			# Increment line counter
			l += 1
			
	def getWBData(self):
		return self.wboutput
	
	def writeWBDataToFile(self,filename=""):
		txt = ''
		try:
			with open(self.default_path+filename,"w") as f:
				pickle.dump(self.wboutput, f)
			txt = "File %s generated succesfully"%(self.default_path+filename)
		except Exception, e:
			txt = "Error while writing the output file: "+str(e)
		return txt

def generate_file(filename):

	if not filename.endswith(".wb"):
		print "\nIncorrect filename selected: %s"%filename
	else:
		gen = Generator()
		
		gen.readWBFile(filename)
		
		# 1.- Build the devine name 
		try:
			vend =  gen.getInputVendorID()
			if vend is None:
				sys.exit()
			prod = gen.getInputProduct()
			if prod is None:
				sys.exit()
			devFileName = "%016x:%08x"%(vend,prod)
		except Exception, e:
			devFileName = None
			print str(e)
			
		if devFileName is not None:
			# 2. Read the wb file
			gen.processWBData()
			
			# 3. Write output to file
			txt = gen.writeWBDataToFile(devFileName)

			print "\nFile %s generated succesfully!!\n"%devFileName
			print txt
		else:
			print "Incorrect filename generated!!", filename

if __name__ == "__main__":
	import sys, os.path
	
	args = sys.argv[1:]
	
	if len(args)>1:
		for el in args:
			if os.path.isfile(el):
				generate_file(el)
	elif os.path.isdir(args[0]):
		arch = os.listdir(args[0])
		for el in arch:
			generate_file(el)
	else:
		if os.path.isfile(args[0]):
			generate_file(args[0])

	sys.exit()
