#----------------------------------------------------------------------------------------------------------------------
#   
#	ctemsdb.py file
#
#	File used to read SDB information form spec card
#	History:
#   30/03/2015 - file created
#
#----------------------------------------------------------------------------------------------------------------------
__author__ = "Manolo Broseta"
__copyright__ = "Copyright 2015, ALBA"
__license__ = "GPLv3 or later"
__version__ = "1.0"
__email__ = "mbroseta@cells.es"
__status__ = "Development"

import os.path
import pickle

from alin import *

__OFFSET__ = 0x00 #0x100
__DEVICE_FOLDER__ = "/home/projects/alba-em/deviceslib/"


def showDevice(devfile=""):
	dev_map = {}
	output = open(devfile, 'rb')
	dev_map = pickle.load(output)
	return dev_map

def help():
	print "ALIN usage as follows:\n"
	print "\t# alin <command_1>=<value> <command_2>=<value>....."
	print "\nCommands are optional. By defult, short SDB structure info is shown."
	print "List of available commands are:\n"
	print "\taddress:\tContinuous read of a memory address. This read will be updated every second until 'enter' key will be pressed.\n"
	print "\tdevicemap:\tShows the available devices list and lets the user to select which device memory map to show.\n"
	print "\tinfo:\t\tTo display the detailed SDB structure.\n"
	print "\thelp:\t\tshows this help\n"
	print "\tload:\t\tTo load a new binary file into the Spec FPGA."
	print "\t\t\tThe file location has to be specified, i.e.: alin load='/lib/firmware/fmc/spec-init.bin'.\n"
	print "\toffset:\t\tTo define the address position where the SDB magic number is located. By defult offset values is 0x100.\n"
	print "\trange:\t\tThis shows an address range of memary. It has to be specified the init address and the end address."
	print "\t\t\tI.e.: alin range init=0x100 end=0x11f\n"
	print "\twrite:\t\tThis commands lets write a value in a certain position."
	print "\t\t\tI.e.: alin write=0x100 data=0x01\n"
	print "\tdev:\t\tThis commands shows the list of registers for a particular device, if previously the device pickle file has been generated using the alingen tool"
	print "\t\t\tThe device name have to be between quotes with the following format VendorId:Product. I.e.: alin dev='alba:609'"
	print "\t\t\tOther options available for a device are:"
	print "\t\t\t\tR: Reads a regsiter. I.e.: alin dev='<VendorID:Product>' R=<REG_NAME>"
	print "\t\t\t\tW: Write and then 'data=' to enter the ex value to write. I.e.: alin dev='<VendorID:Product>' W=<REG_NAME> data=0x<hex_value>"
	print "\t\t\t\tI: Displays complete register info I.e.: alin dev='<VendorID:Product>' I=<REG_NAME>"
	print "\n"

if __name__ == "__main__":
	import sys, os.path
	import select
	args = ' '.join(sys.argv[1:])

	if 'help' in args:
		# Shows help command
		help()
		sys.exit()
	else:
		# Offset commnad
		offset = __OFFSET__
		if 'offset' in args:
			for el in args.split(' '):
				if "offset=" in el:
					offset = int(el.split('=')[1],16)
		
		app = alinSDB(offset)
		
		# Load new file command
		if 'load' in args:
			filename = ''
			for el in args.split(' '):
				if "load=" in el:
					filename = (el.split('=')[1])
			if os.path.isfile(filename):
				ret = app.SDB_loadFile(filename)
				if ret:
					print('the bitstream has been successfully loaded')
				else:
					print('the bitstream loading has failed or cannot find the bitstream!!!')

		app.SDB_readData(offset)

		# Continuous read of an address
		if 'address' in args:
			address = __OFFSET__
			for el in args.split(' '):		
				if 'address=' in el:
					address=int(el.split('=')[1],16)
			
			print "\nPress 'Enter' to stop reading the address:\n"

			i = []
			while i==[]:
				value = app.SDB_readAddress(address)
				sys.stdout.write("\rSDB Read data: Address=%s Value=%s"%(hex(address),format(value,'#010x')))
				sys.stdout.flush()
				i, o, e = select.select( [sys.stdin], [], [], 1 )
			sys.exit()
			
		if 'write' in args:
			address = __OFFSET__
			data = 0
			for el in args.split(' '):		
				print el
				if 'write=' in el:
					address=int(el.split('=')[1],16)
				if 'data=' in el:
					data=int(el.split('=')[1],16)
					
			if address != __OFFSET__:
				app.SDB_writeAddress(address,data)
			sys.exit()

		# Display range of addresses
		if 'range' in args:
			init_address_found = False
			end_address_found = False
			for el in args.split(' '):		
				if 'init=' in el:
					init_address_found = True
					init_address=int(el.split('=')[1],16)			
				if 'end=' in el:
					end_address_found = True
					end_address=int(el.split('=')[1],16)
			
			if not init_address_found:
				init_address = __OFFSET__
				if end_address_found:
					end_address = init_address+4
			if not end_address_found:
				end_address = init_address+4
				
			rd_blk = []
			rd_blk = app.SDB_readAddressRange(init_address,end_address)
			for el in rd_blk:
				print "%06x"%el[0],
				print ' '.join([("%02x"%el[a]) for a in range(1,17)]),
				print " >",''.join([chr(el[a]) if el[a]>0x1F and el[a]<0x80 else chr(0x2E) for a in range(1,17)]),"<"
			sys.exit()


		if 'dev=' in args:
			devname = ''			

			action = None
			for el in args.split(' '):
				if "dev=" in el:
					devname = (el.split('=')[1]).lower()
					if ":" not in devname:
						print "\nIncorret device name format!!!\n"
						sys.exit()
					vd, pd = devname.split(":")
					vd = int(vd.lower(),16)
					pd = int(pd.lower(),16)
					devname = "%016x:%08x"%(vd,pd)
				elif "R=" in el or "r=" in el:
					action = "READ"
					regname = (el.split('=')[1])
				elif "W=" in el or "w=" in el:
					action = "WRITE"
					regname = (el.split('=')[1])
				elif "data=" in el:
					data = int(el.split('=')[1],16)
				elif "I=" in el or "i=" in el:
					action = "INFO"
					regname = (el.split('=')[1])
			
			device = alinDevice(devname)
			if action == "READ":
				value = device.readAttribute(regname)
				txt = "%s "%regname.upper()
				txt = txt.ljust(30,'.')
				txt += ": %s"%hex(value)
				print "\n"+txt+"\n"
			elif action == "WRITE":
				device.writeAttribute(regname,data)
				
				value = device.readAttribute(regname)
				txt = "%s "%regname.upper()
				txt = txt.ljust(30,'.')
				txt += ": %s"%hex(value)
				print "\n"+txt+"\n"
			elif action == "INFO":
				temp = {}
				temp = device.getAttributeInfo(regname)
				try:
					print "\nDescriptrion:"
					dp = temp['desc'].split("\\n")
					for el in dp:
						print "\t",el
				except:
					print "\n%s\n"%temp['desc']
				print "\nAddress: %s"%hex(temp['address'])
				print "Bit position: %s"%str(temp['position'])
				print "Size: %s"%str(temp['size'])
				print "Type: %s"%temp['access']
				txt = "%s "%regname.upper()
				txt = txt.ljust(30,'.')
				txt += ": %s"%hex(temp['value'])
				print "\n"+txt+"\n"
			else:
				devinfo = {}
				devinfo = device.getDeviceInfo()
				
				temp = []
				temp = device.getDeviceData()
				
				print "\nDevice: %s"%devname.upper()
				print "Description: %s\n"%devinfo['description']
				
				for el in temp:
					txt = "%s "%el[0]
					txt = txt.ljust(30,'.')
					txt += ": %s"%el[1]
					print txt
			print "\n"

			sys.exit()

		# Display device memory map
		if 'devicemap' in args:
			numf = False
			rd_blk = []
			for el in args.split(' '):		
				if 'num=' in el:
					numf = True
					dv = data=int(el.split('=')[1])
					rd_blk = app.SDB_showDeviceMemory(dv)					
			if not numf:
				print "Select device from the list:\n"
				#app.SDB_showDeviceList()
				app.SDB_showTree()
				dv = raw_input("\nSelect device number to show (any other key to exit):")
				rd_blk = app.SDB_showDeviceMemory(dv)
			
			if rd_blk is not None:
				for el in rd_blk:
					print "%06x"%el[0],
					print ' '.join([("%02x"%el[a]) for a in range(1,17)]),
					print " >",''.join([chr(el[a]) if el[a]>0x1F and el[a]<0x80 else chr(0x2E) for a in range(1,17)]),"<"
		else:
			# Show sdb in tree mode or long mode instead
			if 'info' in args:
				app.SDB_showInfo()
			elif 'tree' in args:
				app.SDB_showTree()
			else:
				app.SDB_showTree()
					
	sys.exit()